
' Overflow check is disabled in the project's compilation properties.
' Command-line argument validation is not implemented.  However, the
' implementation will only perform an operation if sufficient arguments are
' passed to the tool.
Option Compare Binary
Option Explicit On
Option Strict On

Imports System.IO
Imports System.Xml

Friend Module Program
    Private m_CmdArgs As Dictionary(Of String, String)

    Public Sub Main(ByVal Args() As String)

        ' Argument handling.
        InitializeCmdArgs(Args)
        If Not CmdArgs.ContainsKey("nologo") Then
            WriteCopyright()
        End If
        If CmdArgs.ContainsKey("?") Then
            WriteHelp()
            Return
        End If

        ' Creating output directories and files.
        Dim TargetDirectory As String = Nothing
        If CmdArgs.ContainsKey("out") Then
            TargetDirectory = CmdArgs("out")
            MakeSureDirectoryExists(TargetDirectory)
        End If

        ' Manifest.
        If CmdArgs.ContainsKey("manifest") Then

            ' Building manifest name.
            Dim ManifestFileName As String = CmdArgs("manifest")
            Dim Identity As New AssemblyIdentity()
            Try
                Identity.Load(ManifestFileName)
            Catch ex As Exception
                Console.WriteLine("Error:  Loading manifest file ""{0}"" failed.  ({1})", ManifestFileName, ex.Message)
                Return
            End Try
            Dim ManifestName As String = Identity.GetName()
            Console.WriteLine("Manifest name:  """ & ManifestName & """.")

            ' Creating directories and copying/renaming files.
            If TargetDirectory IsNot Nothing Then
                Dim TargetManifestDirectory As String = Path.Combine(TargetDirectory, "Manifests")
                MakeSureDirectoryExists(TargetManifestDirectory)
                CopyFile(ManifestFileName, Path.Combine(TargetManifestDirectory, ManifestName & ".manifest"))
                If CmdArgs.ContainsKey("manifestcat") Then
                    Dim ManifestCatalogFileName As String = CmdArgs("manifestcat")
                    CopyFile(ManifestCatalogFileName, Path.Combine(TargetManifestDirectory, ManifestName & ".cat"))
                End If

                ' Read file list from manifest and copy files.
                If CmdArgs.ContainsKey("files") Then
                    Dim FileDirectory As String = CmdArgs("files")
                    Dim TargetFileDirectory As String = Path.Combine(TargetDirectory, ManifestName)
                    MakeSureDirectoryExists(TargetFileDirectory)
                    Dim d As New XmlDocument()
                    d.Load(ManifestFileName)
                    Dim nm As New XmlNamespaceManager(d.NameTable)
                    nm.AddNamespace("asm.v1", "urn:schemas-microsoft-com:asm.v1")
                    For Each Node As XmlNode In d.DocumentElement.SelectNodes("/asm.v1:assembly/asm.v1:file/@name", nm)
                        CopyFile(Path.Combine(FileDirectory, Node.Value), Path.Combine(TargetFileDirectory, Node.Value))
                    Next Node
                End If
            End If
        End If

        ' Policy.
        If CmdArgs.ContainsKey("policy") Then

            ' Building policy name.
            Dim PolicyFileName As String = CmdArgs("policy")
            Dim Identity As New AssemblyIdentity()
            Try
                Identity.Load(PolicyFileName)
            Catch ex As Exception
                Console.WriteLine("Error: Loading policy file ""{0}"" failed.  ({1})", PolicyFileName, ex.Message)
                Return
            End Try
            Dim PolicyName As String = Identity.GetName()
            Console.WriteLine("Policy name:  """ & PolicyName & """.")

            ' Creating directories and copying/renaming files.
            If TargetDirectory IsNot Nothing Then
                Dim TargetPolicyDirectory As String = Path.Combine(Path.Combine(TargetDirectory, "Policies"), PolicyName)
                MakeSureDirectoryExists(TargetPolicyDirectory)
                CopyFile(PolicyFileName, Path.Combine(TargetPolicyDirectory, Identity.Attributes("version").Value & ".policy"))
                If CmdArgs.ContainsKey("policycat") Then
                    Dim PolicyCatalogFileName As String = CmdArgs("policycat")
                    CopyFile(PolicyCatalogFileName, Path.Combine(TargetPolicyDirectory, Identity.Attributes("version").Value & ".cat"))
                End If
            End If
        End If
    End Sub

    Private Sub MakeSureDirectoryExists(ByVal Path As String)
        If Directory.Exists(Path) Then
            Return
        End If
        Try
            Directory.CreateDirectory(Path)
        Catch ex As Exception
            Console.WriteLine("Error:  Creating directory ""{0}"" failed.  ({1})", Path, ex.Message)
            Return
        End Try
        Console.WriteLine("Directory ""{0}"" created successfully.", Path)
    End Sub

    Private Sub CopyFile(ByVal Source As String, ByVal Destination As String)
        Try
            File.Copy(Source, Destination)
        Catch ex As Exception
            Console.WriteLine("Error:  Copying ""{0}"" to ""{1}"" failed.  ({2})", Source, Destination, ex.Message)
            Return
        End Try
        Console.WriteLine("File ""{0}"" copied to ""{1}"" successfully.", Source, Destination)
    End Sub

    Private Sub WriteCopyright()
        Console.WriteLine( _
            My.Application.Info.ProductName & " Version " & My.Application.Info.Version.ToString() & ControlChars.NewLine & _
            My.Application.Info.Copyright & ControlChars.NewLine & _
            ControlChars.NewLine & _
            My.Application.Info.Description & ControlChars.NewLine _
        )
    End Sub

    Private Sub WriteHelp()
        Console.WriteLine( _
            "Usage:" & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "sxsasmutil [ -manifest:<file name> [ [ -manifestcat:<file name> ]" & ControlChars.NewLine & _
            "         [ -files:<directory name> ] -out:<directory name> ] |" & ControlChars.NewLine & _
            "         -policy:<file name> [ [ -policycat:<file name> ]" & ControlChars.NewLine & _
            "         -out:<directory name> ] | -? [ -nologo ]" & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "Options:" & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "-manifest:<file name>" & ControlChars.NewLine & _
            "         File name of the assembly manifest containing assembly identity" & ControlChars.NewLine & _
            "         information." & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "-manifestcat:<file name>" & ControlChars.NewLine & _
            "         File name of the manifest's catalog file." & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "-files:<directory name>" & ControlChars.NewLine & _
            "         Directory containing the files listed in the manifest's 'file'" & ControlChars.NewLine & _
            "         elements, typically DLL files." & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "-policy:<file name>" & ControlChars.NewLine & _
            "         File name of the policy containing assembly identity information." & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "-policycat:<file name>" & ControlChars.NewLine & _
            "         File name of the policy's catalog file." & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "-out:<directory name>" & ControlChars.NewLine & _
            "         Directory where output should be placed." & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "-nologo  Runs without displaying copyright information." & ControlChars.NewLine & _
            ControlChars.NewLine & _
            "-?       Displays this information." & ControlChars.NewLine _
        )
    End Sub

    Private Sub InitializeCmdArgs(ByVal Args() As String)
        Dim CmdArgs As New Dictionary(Of String, String)()
        For Each Arg As String In Args
            Try
                Arg = Trim(Arg)
                Dim i As Integer = InStr(Arg, ":")
                If i = 0 Then
                    CmdArgs.Add(Mid(Arg, 2), Nothing)
                Else
                    CmdArgs.Add(Mid(Arg, 2, i - 2), Mid(Arg, i + 1))
                End If
            Catch
            End Try
        Next Arg
        m_CmdArgs = CmdArgs
    End Sub

    Private Property CmdArgs() As Dictionary(Of String, String)
        Get
            Return m_CmdArgs
        End Get
        Set(ByVal Value As Dictionary(Of String, String))
            m_CmdArgs = Value
        End Set
    End Property
End Module